%define anolis_release 4
%bcond_without rpm
%bcond_without bpf
%ifarch ppc ppc64 %{sparc} %{mips} %{riscv}
%bcond_with crash
%else
%bcond_without crash
%endif
%bcond_without dracut
%bcond_without sqlite
%bcond_without monitor
%ifarch %{ix86} x86_64 ppc ppc64 ppc64le aarch64
%bcond_without dyninst
%else
%bcond_with dyninst
%endif
%bcond_without systemd
%bcond_without python3
%bcond_without emacsvim
%bcond_without virthost
%bcond_without sysusers
%bcond_without virtguest
%bcond_without pyparsing
%bcond_without debuginfod
%bcond_without python3_probes
%bcond_with java
%bcond_with docs
%bcond_with httpd
%bcond_with boost
%bcond_with htmldocs
%bcond_with python2_probes
%bcond_with specific_python
%{!?elfutils_version: %global elfutils_version 0.179}
%ifarch x86_64
%bcond_without mokutil
%bcond_without openssl
%else
%bcond_with mokutil
%bcond_with openssl
%endif

%define initdir %{_initddir}

%if %{with virtguest}
   %define udevrulesdir /usr/lib/udev/rules.d
%endif

%define dracutstap %{_prefix}/lib/dracut/modules.d/99stap
%define dracutbindir %{_bindir}

%{!?_rpmmacrodir: %define _rpmmacrodir %{_rpmconfigdir}/macros.d}

# To avoid testsuite/*/*.stp has shebang which doesn't start with '/'
%define __brp_mangle_shebangs_exclude_from .stp$

%define _systemtap_runtime_preinstall \
# See systemd-sysusers(8) sysusers.d(5)\
\
g     stapusr  156\
g     stapsys  157\
g     stapdev  158

%define _systemtap_server_preinstall \
# See systemd-sysusers(8) sysusers.d(5)\
\
g     stap-server  -\
u     stap-server  -      "systemtap compiler server"   /var/lib/stap-server   /sbin/nologin\
m     stap-server stap-server


%define _systemtap_testsuite_preinstall \
# See systemd-sysusers(8) sysusers.d(5)\
\
u     stapusr  -          "systemtap testsuite user"    /   /sbin/nologin\
u     stapsys  -          "systemtap testsuite user"    /   /sbin/nologin\
u     stapdev  -          "systemtap testsuite user"    /   /sbin/nologin\
m     stapusr  stapusr\
m     stapsys  stapusr\
m     stapsys  stapsys\
m     stapdev  stapusr\
m     stapdev  stapdev


Name: systemtap
Version: 5.2
Release: %{anolis_release}%{?release_override}%{?dist}
# for version, see also configure.ac


# Packaging abstract:
#
# systemtap              empty req:-client req:-devel
# systemtap-devel        /usr/bin/stap, runtime, tapset, req:kernel-devel
# systemtap-client       /usr/bin/stap, samples, docs, tapset(bonus), req:-runtime
# systemtap-server       /usr/bin/stap-server*, req:-devel
# systemtap-runtime      /usr/bin/staprun, /usr/bin/stapsh, /usr/bin/stapdyn
# systemtap-sdt-devel    /usr/include/sys/sdt.h AND /usr/bin/dtrace
# systemtap-sdt-dtrace   /usr/bin/dtrace
# systemtap-testsuite    /usr/share/systemtap/testsuite*, req:systemtap, req:sdt-devel
# systemtap-initscript   /etc/init.d/systemtap, dracut module, req:systemtap
# systemtap-runtime-java libHelperSDT.so, HelperSDT.jar, stapbm, req:-runtime
# systemtap-runtime-python2 HelperSDT python2 module, req:-runtime
# systemtap-runtime-python3 HelperSDT python3 module, req:-runtime
# systemtap-runtime-virthost  /usr/bin/stapvirt, req:libvirt req:libxml2
# systemtap-runtime-virtguest udev rules, init scripts/systemd service, req:-runtime
#
# Typical scenarios:
#
# stap-client:           systemtap-client
# stap-server:           systemtap-server
# local user:            systemtap
#
# Unusual scenarios:
#
# intermediary stap-client for --remote:       systemtap-client (-runtime unused)
# intermediary stap-server for --use-server:   systemtap-server (-devel unused)

Summary: Programmable system-wide instrumentation system
License: GPL-2.0-or-later
URL: http://sourceware.org/systemtap/
Source: https://sourceware.org/ftp/systemtap/releases/%{name}-%{version}.tar.gz
Patch001: 0001-add-loongarch64-support-for-systemtap.patch
Patch002: 0002-PR32302-Emit-forward-decls-within-tracepoint_derived.patch

# Build*
BuildRequires: make gcc-c++
BuildRequires: cpio
BuildRequires: gettext-devel
BuildRequires: pkgconfig(nss)
BuildRequires: pkgconfig(avahi-client)
%if %{with debuginfod}
BuildRequires: pkgconfig(libdebuginfod)
%endif
%if %{with dyninst}
BuildRequires: dyninst-devel >= 10.0
BuildRequires: pkgconfig(libselinux)
%endif
%if %{with sqlite}
BuildRequires: sqlite-devel > 3.7
%endif
%if %{with monitor}
BuildRequires: pkgconfig(json-c)
BuildRequires: pkgconfig(ncurses)
%endif
%if %{with systemd}
BuildRequires: systemd
%endif
# Needed for libstd++ < 4.0, without <tr1/memory>
%if %{with boost}
BuildRequires: boost-devel
%endif
%if %{with crash}
BuildRequires: crash-devel zlib-devel
%endif
%if %{with rpm}
BuildRequires: rpm-devel
%endif
BuildRequires: elfutils-devel >= %{elfutils_version}
%if %{with docs}
BuildRequires: /usr/bin/latex /usr/bin/dvips /usr/bin/ps2pdf
BuildRequires: tex(fullpage.sty) tex(fancybox.sty) tex(bchr7t.tfm) tex(graphicx.sty)
%if %{with htmldocs}
# On F10, xmlto's pdf support was broken off into a sub-package,
# called 'xmlto-tex'.  To avoid a specific F10 BuildReq, we'll do a
# file-based buildreq on '/usr/share/xmlto/format/fo/pdf'.
BuildRequires: xmlto /usr/share/xmlto/format/fo/pdf
%endif
%endif
%if %{with emacsvim}
# for _emacs_sitelispdir macros etc.
BuildRequires: emacs
%endif
%if %{with java}
BuildRequires: java-devel
%endif
%if %{with virthost}
# BuildRequires: libvirt-devel >= 1.0.2
BuildRequires: pkgconfig(libvirt)
BuildRequires: pkgconfig(libxml-2.0)
%endif
BuildRequires: readline-devel
%if %{with python2_probes}
BuildRequires: python2-devel
BuildRequires: python-setuptools
%endif
%if %{with python3}
BuildRequires: python3
%endif
%if %{with python3_probes}
BuildRequires: python3-devel
BuildRequires: python3-setuptools
%endif
%if %{with specific_python}
BuildRequires: /usr/bin/pathfix.py
%endif

%if %{with httpd}
BuildRequires: libmicrohttpd-devel
BuildRequires: libuuid-devel
%endif
%if %{with sysusers}
BuildRequires:  systemd-rpm-macros
%endif


# Install requirements
Requires: systemtap-client = %{version}-%{release}
Requires: systemtap-devel = %{version}-%{release}

%description
SystemTap is a powerful tool that allows Linux system administrators
to monitor and analyze system behavior. It provides a flexible and
extensible framework for dynamically instrumenting the kernel and
user-space programs, allowing users to monitor and trace system
activity in real-time.

SystemTap is built on top of the Linux kernel's dynamic tracing
capabilities, which are provided by the kernel's tracepoints and
kprobes features. Tracepoints are pre-defined locations in kernel
code where specific events occur, such as a process being scheduled
or a file being opened. Kprobes are similar to tracepoints, but
they allow users to define their own tracepoints within kernel code.

SystemTap provides a high-level scripting language that allows users
to write custom probes to gather data about system events. These probes
are written in a syntax similar to C, with additional constructs for
accessing kernel data structures and performing advanced operations.
The language provides a rich set of functions and macros for writing
complex probes, as well as a powerful data manipulation and analysis
capability.

One of the key benefits of SystemTap is its ability to monitor system
activity with minimal overhead. SystemTap is designed to have a low
impact on system performance, and it provides a number of optimizations
to minimize the impact of its monitoring activities. It uses a just-in-time
compiler to generate optimized code at runtime, and it supports multiple
output formats, including text, binary, and graphical.

SystemTap can be used to monitor a wide range of system activities,
including CPU usage, disk I/O, network traffic, system calls, and kernel
data structures. It can also be used to monitor user-space programs,
allowing users to identify performance bottlenecks and system issues.
Users can use SystemTap to monitor system activity on a per-process basis,
allowing them to identify issues that are specific to individual programs.

SystemTap consists of a set of kernel modules and libraries that allow
it to interact with the running system. These modules allow SystemTap
to read and write data from the kernel, monitor system activity, and create
custom probes. The modules are designed to be lightweight and non-intrusive,
and they can be easily added or removed from the system as needed.

SystemTap provides a rich set of built-in probes that can be used to
gather data about system activity. These probes can be used to monitor
the system's CPU utilization, memory usage, disk I/O activity, network
activity, and other system events. Users can also create custom probes to
gather data specific to their needs. Once a SystemTap script is written,
it can be executed on a running system using the stap command. The script
will start collecting data and outputting it to the terminal or a log file.
The data can be analyzed using various tools, such as awk, sed, and grep.

In summary, SystemTap is a powerful tool for analyzing Linux systems,
providing a flexible and extensible framework for dynamically instrumenting
the kernel and user-space programs. It allows users to monitor and trace
system activity in real-time, with minimal impact on system performance,
and it provides a rich set of built-in probes and a powerful scripting
language for creating custom probes. With SystemTap, system administrators
can diagnose problems, optimize system performance, and gain a deeper
understanding of system behavior.

%package server
Summary: Instrumentation System Server
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap-devel = %{version}-%{release}
Conflicts: systemtap-devel < %{version}-%{release}
Conflicts: systemtap-runtime < %{version}-%{release}
Conflicts: systemtap-client < %{version}-%{release}
Requires: nss coreutils
Requires: zip unzip
Requires(pre): shadow-utils
BuildRequires: nss-devel avahi-devel
%if %{with openssl}
Requires: openssl
%endif
%if %{with systemd}
Requires: systemd
%else
Requires(post): chkconfig
Requires(preun): chkconfig
Requires(preun): initscripts
Requires(postun): initscripts
%endif

%description server
This is the remote script compilation server component of systemtap.
It announces itself to nearby clients with avahi (if available), and
compiles systemtap scripts to kernel objects on their demand.


%package devel
Summary: Programmable system-wide instrumentation system - development headers, tools
License: GPLv2+
URL: http://sourceware.org/systemtap/

Recommends: (kernel-debug-devel if kernel-debug)
Recommends: (kernel-devel if kernel)

Requires: gcc make
# for compiling --runtime=dyninst sripts, need elfutils headers, bz1930973
Requires: elfutils-devel >= %{elfutils_version}

Conflicts: systemtap-client < %{version}-%{release}
Conflicts: systemtap-server < %{version}-%{release}
Conflicts: systemtap-runtime < %{version}-%{release}
# Suggest: kernel-debuginfo

%description devel
This package contains the components needed to compile a systemtap
script from source form into executable (.ko) forms.  It may be
installed on a self-contained developer workstation (along with the
systemtap-client and systemtap-runtime packages), or on a dedicated
remote server (alongside the systemtap-server package).  It includes
a copy of the standard tapset library and the runtime library C files.


%package runtime
Summary: Programmable system-wide instrumentation system - runtime
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires(pre): shadow-utils
Conflicts: systemtap-devel < %{version}-%{release}
Conflicts: systemtap-server < %{version}-%{release}
Conflicts: systemtap-client < %{version}-%{release}

%description runtime
SystemTap runtime contains the components needed to execute
a systemtap script that was already compiled into a module
using a local or remote systemtap-devel installation.


%package client
Summary: Programmable system-wide instrumentation system - client
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: zip unzip
Requires: systemtap-runtime = %{version}-%{release}
Requires: coreutils grep sed unzip zip
Requires: openssh-clients
Conflicts: systemtap-devel < %{version}-%{release}
Conflicts: systemtap-server < %{version}-%{release}
Conflicts: systemtap-runtime < %{version}-%{release}
%if %{with mokutil}
Requires: mokutil
%endif

%description client
This package contains/requires the components needed to develop
systemtap scripts, and compile them using a local systemtap-devel
or a remote systemtap-server installation, then run them using a
local or remote systemtap-runtime.  It includes script samples and
documentation, and a copy of the tapset library for reference.


%package initscript
Summary: Systemtap Initscripts
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap = %{version}-%{release}
%if %{with systemd}
Requires: systemd
%else
Requires(post): chkconfig
Requires(preun): chkconfig
Requires(preun): initscripts
Requires(postun): initscripts
%endif

%description initscript
This package includes a SysVinit script to launch selected systemtap
scripts at system startup, along with a dracut module for early
boot-time probing if supported.


%package sdt-devel
Summary: Static probe support header files
License: GPL-2.0-or-later AND CC0-1.0
URL: http://sourceware.org/systemtap/
%if %{with pyparsing}
%if %{with python3}
Requires: python3-pyparsing
%else
Requires: pyparsing
%endif
%endif

%description sdt-devel
This package includes the <sys/sdt.h> header file used for static
instrumentation compiled into userspace programs.

# https://inbox.sourceware.org/systemtap/Zy640M7FbVZf-lwz@elastic.org/T/#u
%package sdt-dtrace
Summary: Static probe support dtrace tool
License: GPL-2.0-or-later AND CC0-1.0
URL: https://sourceware.org/systemtap/
Provides: dtrace = %{version}-%{release}
%if %{with_pyparsing}
%if %{with_python3}
Requires: python3-pyparsing
%else
Requires: pyparsing
%endif
%endif

%description sdt-dtrace
This package includes the dtrace-compatibility preprocessor
to process related .d files into tracing-macro-laden .h headers.

%package testsuite
Summary: Instrumentation System Testsuite
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap = %{version}-%{release}
Requires: systemtap-sdt-devel = %{version}-%{release}
Requires: systemtap-server = %{version}-%{release}
Requires: dejagnu which elfutils grep nc
%if %{with debuginfod}
Requires: elfutils-debuginfod
%endif
Requires: gcc gcc-c++ make glibc-devel
# testsuite/systemtap.base/ptrace.exp needs strace
Requires: strace
# testsuite/systemtap.base/ipaddr.exp needs nc. Unfortunately, the rpm
# that provides nc has changed over time (from 'nc' to
# 'nmap-ncat'). So, we'll do a file-based require.
Requires: /usr/bin/nc
# testsuite/systemtap.server/client.exp needs avahi
Requires: avahi
%if %{with crash}
# testsuite/systemtap.base/crash.exp needs crash
Requires: crash
%endif
%if %{with java}
Requires: systemtap-runtime-java = %{version}-%{release}
%endif
%if %{with python2_probes}
Requires: systemtap-runtime-python2 = %{version}-%{release}
%endif
%if %{with python3_probes}
Requires: systemtap-runtime-python3 = %{version}-%{release}
%endif
%ifarch x86_64
# fweimer, personal correspondence
Recommends: glibc-devel(x86-32)
# ... and /usr/lib/libgcc_s.so.*
# ... and /usr/lib/libstdc++.so.*
%endif
# The following "meta" files for the systemtap examples run "perf":
#   testsuite/systemtap.examples/hw_watch_addr.meta
#   testsuite/systemtap.examples/memory/hw_watch_sym.meta
Requires: perf

%description testsuite
This package includes the dejagnu-based systemtap stress self-testing
suite.  This may be used by system administrators to thoroughly check
systemtap on the current system.


%if %{with java}
%package runtime-java
Summary: Systemtap Java Runtime Support
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap-runtime = %{version}-%{release}
Requires: byteman > 2.0
Requires: iproute
Requires: java-devel

%description runtime-java
This package includes support files needed to run systemtap scripts
that probe Java processes running on the OpenJDK runtimes using Byteman.
%endif

%if %{with python2_probes}
%package runtime-python2
Summary: Systemtap Python 2 Runtime Support
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap-runtime = %{version}-%{release}

%description runtime-python2
This package includes support files needed to run systemtap scripts
that probe python 2 processes.
%endif

%if %{with python3_probes}
%package runtime-python3
Summary: Systemtap Python 3 Runtime Support
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap-runtime = %{version}-%{release}

%if ! (%{with python2_probes})
# Provide an clean upgrade path when the python2 package is removed
Obsoletes: %{name}-runtime-python2 < %{version}-%{release}
%endif

%description runtime-python3
This package includes support files needed to run systemtap scripts
that probe python 3 processes.
%endif

%if %{with python3_probes}
%package exporter
Summary: Systemtap-prometheus interoperation mechanism
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap-runtime = %{version}-%{release}

%description exporter
This package includes files for a systemd service that manages
systemtap sessions and relays prometheus metrics from the sessions
to remote requesters on demand.
%endif

%if %{with virthost}
%package runtime-virthost
Summary: Systemtap Cross-VM Instrumentation - host
License: GPLv2+
URL: http://sourceware.org/systemtap/
# only require libvirt-libs really
#Requires: libvirt >= 1.0.2
Requires: libxml2

%description runtime-virthost
This package includes the components required to run systemtap scripts
inside a libvirt-managed domain from the host without using a network
connection.
%endif

%if %{with virtguest}
%package runtime-virtguest
Summary: Systemtap Cross-VM Instrumentation - guest
License: GPLv2+
URL: http://sourceware.org/systemtap/
Requires: systemtap-runtime = %{version}-%{release}
%if %{with systemd}
Requires(post): findutils coreutils
Requires(preun): grep coreutils
Requires(postun): grep coreutils
%else
Requires(post): chkconfig initscripts
Requires(preun): chkconfig initscripts
Requires(postun): initscripts
%endif

%description runtime-virtguest
This package installs the services necessary on a virtual machine for a
systemtap-runtime-virthost machine to execute systemtap scripts.
%endif

%package jupyter
Summary: ISystemtap jupyter kernel and examples
License: GPL-2.0-or-later
URL: https://sourceware.org/systemtap/
Requires: systemtap = %{version}-%{release}

%description jupyter
This package includes files needed to build and run
the interactive systemtap Jupyter kernel, either locally
or within a container.

# ------------------------------------------------------------------------

%prep
%autosetup -p1

%build

# Enable/disable the dyninst pure-userspace backend
%if %{with dyninst}
%global dyninst_config --with-dyninst
%else
%global dyninst_config --without-dyninst
%endif

# Enable/disable the sqlite coverage testing support
%if %{with sqlite}
%global sqlite_config --enable-sqlite
%else
%global sqlite_config --disable-sqlite
%endif

%if %{with debuginfod}
%global debuginfod_config --with-debuginfod
%else
%global debuginfod_config --without-debuginfod
%endif


# Enable/disable the crash extension
%if %{with crash}
%global crash_config --enable-crash
%else
%global crash_config --disable-crash
%endif

# Enable/disable the code to find and suggest needed rpms
%if %{with rpm}
%global rpm_config --with-rpm
%else
%global rpm_config --without-rpm
%endif

%if %{with docs}
%if %{with htmldocs}
%global docs_config --enable-docs --enable-htmldocs
%else
%global docs_config --enable-docs --disable-htmldocs
%endif
%else
%global docs_config --enable-docs=prebuilt
%endif

%if %{with java}
%global java_config --with-java=%{_jvmdir}/java
%else
%global java_config --without-java
%endif

%if %{with python3}
%global python3_config --with-python3
%else
%global python3_config --without-python3
%endif
%if %{with python2_probes}
%global python2_probes_config --with-python2-probes
%else
%global python2_probes_config --without-python2-probes
%endif
%if %{with python3_probes}
%global python3_probes_config --with-python3-probes
%else
%global python3_probes_config --without-python3-probes
%endif

%if %{with virthost}
%global virt_config --enable-virt
%else
%global virt_config --disable-virt
%endif

%if %{with dracut}
%global dracut_config --with-dracutstap=%{dracutstap} --with-dracutbindir=%{dracutbindir}
%else
%global dracut_config %{nil}
%endif

%if %{with httpd}
%global httpd_config --enable-httpd
%else
%global httpd_config --disable-httpd
%endif

%if %{with bpf}
%global bpf_config --with-bpf
%else
%global bpf_config --without-bpf
%endif

# We don't ship compileworthy python code, just oddball samples
%global py_auto_byte_compile 0

%configure %{dyninst_config} %{sqlite_config} %{crash_config} %{docs_config} %{rpm_config} %{java_config} %{virt_config} %{dracut_config} %{python3_config} %{python2_probes_config} %{python3_probes_config} %{httpd_config} %{bpf_config} %{debuginfod_config} --disable-silent-rules --with-extra-version="rpm %{version}-%{release}"
%make_build


%install
%make_install

%if ! (%{with python3})
rm -v $RPM_BUILD_ROOT%{_bindir}/stap-profile-annotate
%endif

%find_lang %{name}
for dir in $(ls -1d $RPM_BUILD_ROOT%{_mandir}/{??,??_??}) ; do
    dir=$(echo $dir | sed -e "s|^$RPM_BUILD_ROOT||")
    lang=$(basename $dir)
    echo "%%lang($lang) $dir/man*/*" >> %{name}.lang
done

%if %{with sysusers}
mkdir -p %{buildroot}%{_sysusersdir}
echo '%_systemtap_runtime_preinstall' > %{buildroot}%{_sysusersdir}/systemtap-runtime.conf
echo '%_systemtap_server_preinstall' > %{buildroot}%{_sysusersdir}/systemtap-server.conf
echo '%_systemtap_testsuite_preinstall' > %{buildroot}%{_sysusersdir}/systemtap-testsuite.conf
%endif


ln -s %{_datadir}/systemtap/examples

# Fix paths in the example scripts.
find $RPM_BUILD_ROOT%{_datadir}/systemtap/examples -type f -name '*.stp' -print0 | xargs -0 sed -i -r -e '1s@^#!.+stap@#!%{_bindir}/stap@'

# To make rpmlint happy, remove any .gitignore files in the testsuite.
find testsuite -type f -name '.gitignore' -print0 | xargs -0 rm -f

# Because "make install" may install staprun with whatever mode, the
# post-processing programs rpmbuild runs won't be able to read it.
# So, we change permissions so that they can read it.  We'll set the
# permissions back to 04110 in the %files section below.
chmod 755 $RPM_BUILD_ROOT%{_bindir}/staprun

#install the useful stap-prep script
install -c -m 755 stap-prep $RPM_BUILD_ROOT%{_bindir}/stap-prep

# Copy over the testsuite
cp -rp testsuite $RPM_BUILD_ROOT%{_datadir}/systemtap

# We want the manuals in the special doc dir, not the generic doc install dir.
# We build it in place and then move it away so it doesn't get installed
# twice. rpm can specify itself where the (versioned) docs go with the
# %doc directive.
mkdir docs.installed
mv $RPM_BUILD_ROOT%{_datadir}/doc/systemtap/*.pdf docs.installed/
%if %{with docs}
%if %{with htmldocs}
mv $RPM_BUILD_ROOT%{_datadir}/doc/systemtap/tapsets docs.installed/
mv $RPM_BUILD_ROOT%{_datadir}/doc/systemtap/SystemTap_Beginners_Guide docs.installed/
%endif
%endif

install -D -m 644 macros.systemtap $RPM_BUILD_ROOT%{_rpmmacrodir}/macros.systemtap

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/stap-server
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/logrotate.d
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/run/systemtap
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/lib/stap-server
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/log/stap-server
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/cache/systemtap
touch $RPM_BUILD_ROOT%{_localstatedir}/log/stap-server/log
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/lib/stap-server/.systemtap
install -m 644 initscript/logrotate.stap-server $RPM_BUILD_ROOT%{_sysconfdir}/logrotate.d/stap-server

# If using systemd systemtap.service file, retain the old init script in %{_libexecdir} as a helper.
%if %{with systemd}
mkdir -p $RPM_BUILD_ROOT%{_unitdir}
touch $RPM_BUILD_ROOT%{_unitdir}/systemtap.service
install -m 644 initscript/systemtap.service $RPM_BUILD_ROOT%{_unitdir}/systemtap.service
mkdir -p $RPM_BUILD_ROOT%{_sbindir}
install -m 755 initscript/systemtap $RPM_BUILD_ROOT%{_sbindir}/systemtap-service
%else
mkdir -p $RPM_BUILD_ROOT%{initdir}
install -m 755 initscript/systemtap $RPM_BUILD_ROOT%{initdir}
mkdir -p $RPM_BUILD_ROOT%{_sbindir}
ln -sf %{initdir}/systemtap $RPM_BUILD_ROOT%{_sbindir}/systemtap-service
# TODO CHECK CORRECTNESS: symlink %{_sbindir}/systemtap-service to %{initdir}/systemtap
%endif

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/systemtap
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/systemtap/conf.d
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/systemtap/script.d
install -m 644 initscript/config.systemtap $RPM_BUILD_ROOT%{_sysconfdir}/systemtap/config

%if %{with systemd}
mkdir -p $RPM_BUILD_ROOT%{_unitdir}
mkdir -p $RPM_BUILD_ROOT%{_tmpfilesdir}
touch $RPM_BUILD_ROOT%{_unitdir}/stap-server.service
install -m 644 stap-server.conf $RPM_BUILD_ROOT%{_tmpfilesdir}/stap-server.conf
install -m 644 stap-server.service $RPM_BUILD_ROOT%{_unitdir}/stap-server.service
%else
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/stap-server/conf.d
install -m 755 initscript/stap-server $RPM_BUILD_ROOT%{initdir}
install -m 644 initscript/config.stap-server $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig/stap-server
%endif

%if %{with emacsvim}
mkdir -p $RPM_BUILD_ROOT%{_emacs_sitelispdir}
mkdir -p $RPM_BUILD_ROOT%{_emacs_sitestartdir}
install -p -m 644 emacs/systemtap-mode.el* $RPM_BUILD_ROOT%{_emacs_sitelispdir}
install -p -m 644 emacs/systemtap-init.el $RPM_BUILD_ROOT%{_emacs_sitestartdir}/systemtap-init.el
for subdir in ftdetect ftplugin indent syntax
do
    mkdir -p $RPM_BUILD_ROOT%{_datadir}/vim/vimfiles/$subdir
    install -p -m 644 vim/$subdir/*.vim $RPM_BUILD_ROOT%{_datadir}/vim/vimfiles/$subdir
done
%endif

%if %{with virtguest}
   mkdir -p $RPM_BUILD_ROOT%{udevrulesdir}
   %if %{with systemd}
      mkdir -p $RPM_BUILD_ROOT%{_unitdir}
      install -p -m 644 staprun/guest/99-stapsh.rules $RPM_BUILD_ROOT%{udevrulesdir}
      install -p -m 644 staprun/guest/stapsh@.service $RPM_BUILD_ROOT%{_unitdir}
   %else
      mkdir -p $RPM_BUILD_ROOT%{_libexecdir}/systemtap
      mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig/modules
      install -p -m 755 staprun/guest/stapshd $RPM_BUILD_ROOT%{initdir}
      install -p -m 644 staprun/guest/99-stapsh-init.rules $RPM_BUILD_ROOT%{udevrulesdir}
      install -p -m 755 staprun/guest/stapsh-daemon $RPM_BUILD_ROOT%{_libexecdir}/systemtap
      install -p -m 755 staprun/guest/virtio_console.modules $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig/modules
   %endif
%endif

%if %{with dracut}
   mkdir -p $RPM_BUILD_ROOT%{dracutstap}
   touch $RPM_BUILD_ROOT%{dracutstap}/params.conf
   install -p -m 755 initscript/99stap/check $RPM_BUILD_ROOT%{dracutstap}
   install -p -m 755 initscript/99stap/install $RPM_BUILD_ROOT%{dracutstap}
   install -p -m 755 initscript/99stap/module-setup.sh $RPM_BUILD_ROOT%{dracutstap}
   install -p -m 755 initscript/99stap/start-staprun.sh $RPM_BUILD_ROOT%{dracutstap}
%endif

%if %{with specific_python}
# Some files got ambiguous python shebangs, we fix them after everything else is done
pathfix.py -pni "%{__python3} %{py3_shbang_opts}" %{buildroot}%{python3_sitearch} %{buildroot}%{_bindir}/*
%endif

%pre runtime
%if %{with sysusers}
echo '%_systemtap_runtime_preinstall' | systemd-sysusers --replace=%{_sysusersdir}/systemtap-runtime.conf -
%else
getent group stapusr >/dev/null || groupadd -f -g 156 -r stapusr
getent group stapsys >/dev/null || groupadd -f -g 157 -r stapsys
getent group stapdev >/dev/null || groupadd -f -g 158 -r stapdev
%endif
exit 0

%pre server
%if %{with sysusers}
echo '%_systemtap_server_preinstall' | systemd-sysusers --replace=%{_sysusersdir}/systemtap-server.conf -
%else
getent group stap-server >/dev/null || groupadd -f -g 155 -r stap-server
getent passwd stap-server >/dev/null || \
  useradd -c "Systemtap Compile Server" -u 155 -g stap-server -d %{_localstatedir}/lib/stap-server -r -s /sbin/nologin stap-server 2>/dev/null || \
  useradd -c "Systemtap Compile Server" -g stap-server -d %{_localstatedir}/lib/stap-server -r -s /sbin/nologin stap-server
%endif
exit 0

%pre testsuite
%if %{with sysusers}
echo '%_systemtap_testsuite_preinstall' | systemd-sysusers --replace=%{_sysusersdir}/systemtap-testsuite.conf -
%else
getent passwd stapusr >/dev/null || \
    useradd -c "Systemtap 'stapusr' User" -g stapusr -r -s /sbin/nologin stapusr
getent passwd stapsys >/dev/null || \
    useradd -c "Systemtap 'stapsys' User" -g stapsys -G stapusr -r -s /sbin/nologin stapsys
getent passwd stapdev >/dev/null || \
    useradd -c "Systemtap 'stapdev' User" -g stapdev -G stapusr -r -s /sbin/nologin stapdev
%endif
exit 0

%post server

# We have some duplication between the %files listings for the
# ~stap-server directories and the explicit mkdir/chown/chmod bits
# here.  Part of the reason may be that a preexisting stap-server
# account may well be placed somewhere other than
# %{_localstatedir}/lib/stap-server, but we'd like their permissions
# set similarly.

test -e ~stap-server && chmod 750 ~stap-server

if [ ! -f ~stap-server/.systemtap/rc ]; then
  mkdir -p ~stap-server/.systemtap
  chown stap-server:stap-server ~stap-server/.systemtap
  # PR16276: guess at a reasonable number for a default --rlimit-nproc
  numcpu=`/usr/bin/getconf _NPROCESSORS_ONLN`
  if [ -z "$numcpu" -o "$numcpu" -lt 1 ]; then numcpu=1; fi
  nproc=`expr $numcpu \* 30`
  echo "--rlimit-as=614400000 --rlimit-cpu=60 --rlimit-nproc=$nproc --rlimit-stack=1024000 --rlimit-fsize=51200000" > ~stap-server/.systemtap/rc
  chown stap-server:stap-server ~stap-server/.systemtap/rc
fi

test -e %{_localstatedir}/log/stap-server/log || {
     touch %{_localstatedir}/log/stap-server/log
     chmod 644 %{_localstatedir}/log/stap-server/log
     chown stap-server:stap-server %{_localstatedir}/log/stap-server/log
}
# Prepare the service
%if %{with systemd}
     # /bin/systemctl enable stap-server.service >/dev/null 2>&1 || :
     /bin/systemd-tmpfiles --create %{_tmpfilesdir}/stap-server.conf >/dev/null 2>&1 || :
%else
    /sbin/chkconfig --add stap-server
%endif
exit 0

%triggerin client -- systemtap-server
if test -e ~stap-server/.systemtap/ssl/server/stap.cert; then
   # echo Authorizing ssl-peer/trusted-signer certificate for local systemtap-server
   %{_libexecdir}/systemtap/stap-authorize-cert ~stap-server/.systemtap/ssl/server/stap.cert %{_sysconfdir}/systemtap/staprun >/dev/null
   %{_libexecdir}/systemtap/stap-authorize-cert ~stap-server/.systemtap/ssl/server/stap.cert %{_sysconfdir}/systemtap/ssl/client >/dev/null
fi
exit 0
# XXX: corresponding %triggerun?

%preun server
# Check that this is the actual deinstallation of the package, as opposed to
# just removing the old package on upgrade.
if [ $1 = 0 ] ; then
    %if %{with systemd}
       /bin/systemctl --no-reload disable stap-server.service >/dev/null 2>&1 || :
       /bin/systemctl stop stap-server.service >/dev/null 2>&1 || :
    %else
        /sbin/service stap-server stop >/dev/null 2>&1
        /sbin/chkconfig --del stap-server
    %endif
fi
exit 0

%postun server
# Check whether this is an upgrade of the package.
# If so, restart the service if it's running
if [ "$1" -ge "1" ] ; then
    %if %{with systemd}
        /bin/systemctl condrestart stap-server.service >/dev/null 2>&1 || :
    %else
        /sbin/service stap-server condrestart >/dev/null 2>&1 || :
    %endif
fi
exit 0

%post initscript
%if %{with systemd}
    /bin/systemctl enable systemtap.service >/dev/null 2>&1 || :
%else
    /sbin/chkconfig --add systemtap
%endif
exit 0

%preun initscript
# Check that this is the actual deinstallation of the package, as opposed to
# just removing the old package on upgrade.
if [ $1 = 0 ] ; then
    %if %{with systemd}
        /bin/systemctl --no-reload disable systemtap.service >/dev/null 2>&1 || :
        /bin/systemctl stop systemtap.service >/dev/null 2>&1 || :
    %else
        /sbin/service systemtap stop >/dev/null 2>&1
        /sbin/chkconfig --del systemtap
    %endif
fi
exit 0

%postun initscript
# Check whether this is an upgrade of the package.
# If so, restart the service if it's running
if [ "$1" -ge "1" ] ; then
    %if %{with systemd}
        /bin/systemctl condrestart systemtap.service >/dev/null 2>&1 || :
    %else
        /sbin/service systemtap condrestart >/dev/null 2>&1 || :
    %endif
fi
exit 0

%post runtime-virtguest
%if %{with systemd}
   # Start services if there are ports present
   if [ -d /dev/virtio-ports ]; then
      (find /dev/virtio-ports -iname 'org.systemtap.stapsh.[0-9]*' -type l \
         | xargs -n 1 basename \
         | xargs -n 1 -I {} /bin/systemctl start stapsh@{}.service) >/dev/null 2>&1 || :
   fi
%else
   /sbin/chkconfig --add stapshd
   /sbin/chkconfig stapshd on
   /sbin/service stapshd start >/dev/null 2>&1 || :
%endif
exit 0

%preun runtime-virtguest
# Stop service if this is an uninstall rather than an upgrade
if [ $1 = 0 ]; then
   %if %{with systemd}
      # We need to stop all stapsh services. Because they are instantiated from
      # a template service file, we can't simply call disable. We need to find
      # all the running ones and stop them all individually
      for service in `/bin/systemctl --full | grep stapsh@ | cut -d ' ' -f 1`; do
         /bin/systemctl stop $service >/dev/null 2>&1 || :
      done
   %else
      /sbin/service stapshd stop >/dev/null 2>&1
      /sbin/chkconfig --del stapshd
   %endif
fi
exit 0

%postun runtime-virtguest
# Restart service if this is an upgrade rather than an uninstall
if [ "$1" -ge "1" ]; then
   %if %{with systemd}
      # We need to restart all stapsh services. Because they are instantiated from
      # a template service file, we can't simply call restart. We need to find
      # all the running ones and restart them all individually
      for service in `/bin/systemctl --full | grep stapsh@ | cut -d ' ' -f 1`; do
         /bin/systemctl condrestart $service >/dev/null 2>&1 || :
      done
   %else
      /sbin/service stapshd condrestart >/dev/null 2>&1
   %endif
fi
exit 0

%if %{with python3_probes}
%if %{with systemd}
%preun exporter
if [ $1 = 0 ] ; then
  /bin/systemctl stop stap-exporter.service >/dev/null 2>&1 || :
  /bin/systemctl disable stap-exporter.service >/dev/null 2>&1 || :
fi
exit 0

%postun exporter
# Restart service if this is an upgrade rather than an uninstall
if [ "$1" -ge "1" ]; then
   /bin/systemctl condrestart stap-exporter >/dev/null 2>&1 || :
fi
exit 0
%endif
%endif

%post
# Remove any previously-built uprobes.ko materials
(make -C %{_datadir}/systemtap/runtime/uprobes clean) >/dev/null 2>&1 || true
(/sbin/rmmod uprobes) >/dev/null 2>&1 || true

%preun
# Ditto
(make -C %{_datadir}/systemtap/runtime/uprobes clean) >/dev/null 2>&1 || true
(/sbin/rmmod uprobes) >/dev/null 2>&1 || true

# ------------------------------------------------------------------------

%files
# The main "systemtap" rpm doesn't include any files.

%files server -f systemtap.lang
%{_bindir}/stap-server
%{_mandir}/man7/error*
%{_mandir}/man7/warning*
%{_mandir}/man7/stappaths.7*
%{_mandir}/man8/stap-server.8*
%dir %{_libexecdir}/systemtap
%{_libexecdir}/systemtap/stap-env
%{_libexecdir}/systemtap/stap-serverd
%{_libexecdir}/systemtap/stap-gen-cert
%{_libexecdir}/systemtap/stap-stop-server
%{_libexecdir}/systemtap/stap-sign-module
%{_libexecdir}/systemtap/stap-start-server
%{_libexecdir}/systemtap/stap-authorize-cert
%if %{with systemd}
%{_unitdir}/stap-server.service
%{_tmpfilesdir}/stap-server.conf
%else
%{initdir}/stap-server
%dir %{_sysconfdir}/stap-server/conf.d
%config(noreplace) %{_sysconfdir}/sysconfig/stap-server
%endif
%dir %{_sysconfdir}/stap-server
%{!?_licensedir:%global license %%doc}
%doc README README.unprivileged AUTHORS NEWS
%config(noreplace) %{_sysconfdir}/logrotate.d/stap-server
%dir %attr(0750,stap-server,stap-server) %{_localstatedir}/lib/stap-server
%dir %attr(0755,stap-server,stap-server) %{_localstatedir}/log/stap-server
%ghost %attr(0755,stap-server,stap-server) %{_localstatedir}/run/stap-server
%dir %attr(0700,stap-server,stap-server) %{_localstatedir}/lib/stap-server/.systemtap
%ghost %config(noreplace) %attr(0644,stap-server,stap-server) %{_localstatedir}/log/stap-server/log
%license COPYING
%if %{with sysusers}
%{_sysusersdir}/systemtap-server.conf
%endif


%files devel -f systemtap.lang
%{_bindir}/stap
%{_bindir}/stap-prep
%if %{with python3}
%{_bindir}/stap-profile-annotate
%endif
%{_bindir}/stap-report
%dir %{_datadir}/systemtap
%{_datadir}/systemtap/runtime
%{_datadir}/systemtap/tapset
%{_mandir}/man7/error*
%{_mandir}/man1/stap.1*
%{_mandir}/man7/warning*
%{_mandir}/man1/stap-prep.1*
%{_mandir}/man7/stappaths.7*
%{_mandir}/man1/stap-report.1*
%doc README README.unprivileged AUTHORS NEWS
%{!?_licensedir:%global license %%doc}
%license COPYING
%if %{with java}
%dir %{_libexecdir}/systemtap
%{_libexecdir}/systemtap/libHelperSDT.so
%endif
%if %{with emacsvim}
%{_emacs_sitelispdir}/*.el*
%{_datadir}/vim/vimfiles/*/*.vim
%{_emacs_sitestartdir}/systemtap-init.el
%endif
# Notice that the stap-resolve-module-function.py file is used by
# *both* the python2 and python3 subrpms.  Both subrpms use that same
# python script to help list python probes.
%if %{with python3_probes} || %{with python2_probes}
%{_libexecdir}/systemtap/python/stap-resolve-module-function.py
%exclude %{_libexecdir}/systemtap/python/stap-resolve-module-function.py?
%endif


%files runtime -f systemtap.lang
%{_bindir}/stapsh
%{_bindir}/stap-merge
%{_bindir}/stap-report
%if %{with dyninst}
%{_bindir}/stapdyn
%endif
%if %{with bpf}
%{_bindir}/stapbpf
%endif
%if %{with crash}
%dir %{_libdir}/systemtap
%{_libdir}/systemtap/staplog.so*
%endif
%{_mandir}/man7/error*
%{_mandir}/man7/warning*
%{_mandir}/man8/stapsh.8*
%{_mandir}/man8/staprun.8*
%{_mandir}/man7/stappaths.7*
%dir %{_libexecdir}/systemtap
%{_mandir}/man1/stap-report.1*
%{_libexecdir}/systemtap/stapio
%if %{with dyninst}
%{_mandir}/man8/stapdyn.8*
%endif
%if %{with bpf}
%{_mandir}/man8/stapbpf.8*
%endif
%{!?_licensedir:%global license %%doc}
%license COPYING
%if %{with sysusers}
%{_sysusersdir}/systemtap-runtime.conf
%endif
%doc README README.security AUTHORS NEWS
%attr(4110,root,stapusr) %{_bindir}/staprun
%{_libexecdir}/systemtap/stap-authorize-cert

%files initscript
%if %{with systemd}
%{_unitdir}/systemtap.service
%{_sbindir}/systemtap-service
%else
%{initdir}/systemtap
%{_sbindir}/systemtap-service
%endif
%dir %{_sysconfdir}/systemtap
%dir %{_sysconfdir}/systemtap/conf.d
%dir %{_sysconfdir}/systemtap/script.d
%config(noreplace) %{_sysconfdir}/systemtap/config
%dir %{_localstatedir}/cache/systemtap
%ghost %{_localstatedir}/run/systemtap
%{_mandir}/man8/systemtap-service.8*
%if %{with dracut}
   %dir %{dracutstap}
   %{dracutstap}/*
%endif


%files client -f systemtap.lang
%{_bindir}/stap
%license COPYING
%doc docs.installed/*.pdf
%{_datadir}/systemtap/examples
%{!?_licensedir:%global license %%doc}
%doc README README.unprivileged AUTHORS NEWS
%if %{with docs}
%if %{with htmldocs}
%doc docs.installed/tapsets/*.html
%doc docs.installed/SystemTap_Beginners_Guide
%endif
%endif
%{_bindir}/stap-prep
%{_bindir}/stap-report
%{_mandir}/man1/stap.1*
%{_mandir}/man1/stapref.1*
%{_mandir}/man1/stap-prep.1*
%{_mandir}/man1/stap-merge.1*
%{_mandir}/man1/stap-report.1*
%{_mandir}/man3/*
%{_mandir}/man7/error*
%{_mandir}/man7/warning*
%dir %{_datadir}/systemtap
%{_mandir}/man7/stappaths.7*
%{_datadir}/systemtap/tapset

%files sdt-devel
%license COPYING
%{_includedir}/sys/sdt.h
%{_includedir}/sys/sdt-config.h
%{_rpmmacrodir}/macros.systemtap
%{!?_licensedir:%global license %%doc}
%{_bindir}/dtrace
%{_mandir}/man1/dtrace.1*

%files sdt-dtrace
%{_bindir}/dtrace
%doc README AUTHORS NEWS
%{!?_licensedir:%global license %%doc}
%license COPYING
%{_mandir}/man1/dtrace.1*


%if %{with virtguest}
%files runtime-virtguest
%if %{with systemd}
   %{_unitdir}/stapsh@.service
   %{udevrulesdir}/99-stapsh.rules
%else
   %{initdir}/stapshd
   %dir %{_libexecdir}/systemtap
   %{udevrulesdir}/99-stapsh-init.rules
   %{_libexecdir}/systemtap/stapsh-daemon
   %{_sysconfdir}/sysconfig/modules/virtio_console.modules
%endif
%endif

%files testsuite
%dir %{_datadir}/systemtap
%{_datadir}/systemtap/testsuite
%if %{with sysusers}
%{_sysusersdir}/systemtap-testsuite.conf
%endif


%if %{with java}
%files runtime-java
%dir %{_libexecdir}/systemtap
%{_libexecdir}/systemtap/stapbm
%{_libexecdir}/systemtap/HelperSDT.jar
%{_libexecdir}/systemtap/libHelperSDT.so
%endif

%if %{with python3_probes}
%files exporter
%{_sbindir}/stap-exporter
%{_sysconfdir}/stap-exporter
%{_mandir}/man8/stap-exporter.8*
%{_unitdir}/stap-exporter.service
%{_sysconfdir}/sysconfig/stap-exporter
%endif


%if %{with python2_probes}
%files runtime-python2
%{python_sitearch}/HelperSDT
%{python_sitearch}/HelperSDT-*.egg-info
%endif
%if %{with python3_probes}
%files runtime-python3
%{python3_sitearch}/HelperSDT
%{python3_sitearch}/HelperSDT-*.egg-info
%endif

%if %{with virthost}
%files runtime-virthost
%{_bindir}/stapvirt
%{_mandir}/man1/stapvirt.1*
%endif

%files jupyter
%{_bindir}/stap-jupyter-container
%{_bindir}/stap-jupyter-install
%{_mandir}/man1/stap-jupyter.1*
%dir %{_datadir}/systemtap
%{_datadir}/systemtap/interactive-notebook

# ------------------------------------------------------------------------

%changelog
* Sun Apr 27 2025 Shangtong Guo <guo.shangtong@zte.com.cn> - 5.2-4
- add support for riscv64 build

* Fri Apr 11 2025 mgb01105731 <mgb01105731@alibaba-inc.com> - 5.2-3
- Supply /usr/bin/dtrace in sdt-devel subrpm too

* Tue Mar 25 2025 Bo Ren <rb01097748@alibaba-inc.com> - 5.2-2
- Breaks up the sdt.h header file and the /usr/bin/dtrace stub-builder script

* Thu Mar 20 2025 Weisson <Weisson@linux.alibaba.com> - 5.2-1
- Update to 5.2 from 4.8
- add jupyter kernel and examples in jupyter package from 4.9 (
  https://inbox.sourceware.org/systemtap/ZEw5pdqi7y%2FjYqeA@elastic.org/T/#u)
- Emit forward decls within tracepoint_derived_probe_group() too
  [Patches cherry-picked from c10s. Original changelog:
   - Emit forward decls within tracepoint_derived_probe_group() too (5.2-1)]

* Fri Mar 15 2024 Zhao Hang <wb-zh951434@alibaba-inc.com> - 4.8-5
- Rebuild with python3.11

* Fri Nov  3 2023 Wenlong Zhang <zhangwenlong@loongson.cn> - 4.8-4
- disable dyninst for loongarch64

* Fri Apr 14 2023 Xuchun Shang <xuchun.shang@linux.alibaba.com> - 4.8-3
- Update the description

* Mon Mar 6 2023 Funda Wang <fundawang@yeah.net> - 4.8-2
- rebuilt for boost and dyninst

* Thu Jan 12 2023 Funda Wang <fundawang@yeah.net> - 4.8-1
- New version 4.8

* Fri Jul 15 2022 happy_orange <songnannan@linux.alibaba.com> - 4.6-2
- rebuilt for boost and dyninst

* Wed Mar 09 2022 Yuanhong Peng <yummypeng@linux.alibaba.com> - 4.6-1
- Init from upstream v4.6
- Backport some patches to fix compilation error with gcc12
